package cn.ecpark.service.usergw;

import java.util.LinkedHashMap;

import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.context.SpringBootTest.WebEnvironment;
import org.springframework.boot.web.server.LocalServerPort;
import org.springframework.test.context.TestPropertySource;
import org.springframework.test.context.junit4.SpringRunner;

import reactor.netty.http.client.HttpClient;
import reactor.netty.http.client.HttpClient.ResponseReceiver;
import reactor.netty.http.client.HttpClientResponse;



// TODO: 添加附加Dubbo Service for Test启动进程
@SpringBootTest(classes= App.class, webEnvironment = WebEnvironment.RANDOM_PORT)
@RunWith(SpringRunner.class)
@TestPropertySource(locations = {"application.properties"})
public class TestHttp2DubboConfig {

    @LocalServerPort
    private int serverPort;

    @Test
    public void TestRequestHttp2DubboWithoutConfigMethod() {
        
        //  Test Base Url
        HttpClient client = HttpClient.create();
        ResponseReceiver<?> receiver = client.baseUrl("http://localhost:" + serverPort + "/dubbo/hello").headers(
                h -> h.set("method", "Say").add("param-types", "java.lang.String").add("params", "Test-MyHttp2dubbo"))
                .get();

        HttpClientResponse resp = receiver.response().block();
        Assert.assertNotNull(resp);
        String v = resp.responseHeaders().get("X-Response-Default-Foo");
        Assert.assertNotNull(v);
        Assert.assertEquals(v, "Test-Default-Bar");
        
        String content = receiver.responseContent().asString().blockFirst();
        Assert.assertNotNull(content);
        Assert.assertEquals(content, "Test-MyHttp2dubbo");
        
        receiver = client.baseUrl("http://localhost:" + serverPort + "/dubbo/hello").headers(
            h -> h.set("method", "Hello"))
            .get();
        resp = receiver.response().block();
        Assert.assertNotNull(resp);
        content = receiver.responseContent().asString().blockLast();
        Assert.assertNotNull(content);
        Assert.assertEquals(content, "Hello Dubbo");

    }

    @Test
    public void TestJSONReturnWithoutConfigMethod() {
        //  Test Base Url
        HttpClient client = HttpClient.create();
        ResponseReceiver<?> receiver = client.baseUrl("http://localhost:" + serverPort + "/dubbo/hello").headers(
                h -> h.set("method", "Say").add("param-types", "java.lang.String").add("params","{\"a\": 1, \"b\":\"123\"}"))
                .get();

        HttpClientResponse resp = receiver.response().block();
        Assert.assertNotNull(resp);
        String v = resp.responseHeaders().get("X-Response-Default-Foo");
        Assert.assertNotNull(v);
        Assert.assertEquals(v, "Test-Default-Bar");
        
        String content = receiver.responseContent().asString().blockLast();
        Assert.assertNotNull(content);
        Assert.assertEquals(content, "{\"a\": 1, \"b\":\"123\"}");
        
        receiver = client.baseUrl("http://localhost:" + serverPort + "/dubbo/hello").headers(
            h -> h.set("method", "Say").add("param-types", "java.lang.String").add("params","123"))
            .get();
        resp = receiver.response().block();
        Assert.assertNotNull(resp);
        content = receiver.responseContent().asString().blockLast();
        Assert.assertNotNull(content);
        Assert.assertEquals(content, "123");
    }

}